import { cellStyle } from "@/types";
import { Cell, EventArgs, Graph } from "@antv/x6";
import { message } from "antd";
import { useRef, useState } from "react";
import { Selection } from "@repo/x6-plugin-selection";
import { Keyboard } from "@antv/x6-plugin-keyboard";
import { History } from "@antv/x6-plugin-history";
import { Scroller } from "@antv/x6-plugin-scroller";
import { ProjectInfo } from "@/types";
import Topic from "@/components/mindMap/Topic";
import { BorderSize } from "@/enum";

export default function mindMapModel() {
  const [rightToobarActive, setRgithtToolbarActive] = useState<
    "style" | "structure" | "theme" | "icon" | "image" | "tag"
  >();
  // 格式刷启用
  const [enableFormatBrush, setEnableFormatBrush] = useState(false);
  // 格式刷样式
  const formatBrushStyle = useRef<cellStyle>();
  // 脑图实例
  const graphRef = useRef<Graph>();
  const [canRedo, setCanRedo] = useState(false);
  const [canUndo, setCanUndo] = useState(false);
  // 项目信息
  const [mindProjectInfo, setMindProjectInfo] = useState({
    name: "新建脑图",
    desc: "",
    version: "",
    author: "",
  });

  // 初始化脑图
  const initMindMap = (container: HTMLElement) => {
    const instance = new Graph({
      container,
      autoResize: true,
      connecting: {
        connectionPoint: 'anchor',
      },
    });

    instance.use(new Selection());
    instance.use(new Keyboard());
    instance.use(new History());
    instance.use(new Scroller());
    graphRef.current = instance;

    loadProject();

    instance.on('history:change', () => {
      setCanRedo(instance.canRedo());
      setCanUndo(instance.canUndo());
    })
  }

  const loadProject = (projectInfo?: ProjectInfo) => {
    if( !graphRef.current ) return;

    const cells: Cell[] = [];
    if (projectInfo) {
      cells.push(...JSON.parse(projectInfo.data));
    } else {
      const { offsetHeight, offsetWidth } = document.body;
      graphRef.current.size;
      // 默认加载
      cells.push(graphRef.current?.createNode({
        ...Topic,
        root: true,
        data: {
          ...Topic.data,
          fill: {
            ...Topic.data.fill,
            color1: "#30304D"
          },
          text: {
            ...Topic.data.text,
            fontSize: 30,
            color: "#fff"
          },
          label: "未命名文件",
          borderSize: BorderSize.medium
        },
        position: {
          x: offsetWidth / 2,
          y: offsetHeight / 2
        }
      }));
    }
    console.log(cells)
    graphRef.current.addCell(cells);
    graphRef.current.centerCell(cells[0]);
  }

  const handleClick = (args: EventArgs & { cell: Cell }) => {
    // 取消格式刷
    if (!args?.cell || args?.cell?.data?.isPage) {
      formatBrushStyle.current = undefined;
      setEnableFormatBrush(false);
      graphRef.current?.off("cell:click", handleClick);
      graphRef.current?.off("blank:click", handleClick);
    } else {
      if (args.cell.data?.lock) return;
      // 应用格式刷
      const data = args.cell.data;
      args.cell.setData({
        text: formatBrushStyle.current?.text || data?.text,
        fill: formatBrushStyle.current?.fill || data?.fill,
        stroke: formatBrushStyle.current?.stroke || data?.stroke,
        opacity: formatBrushStyle.current?.opacity || data?.opacity,
      });
    }
  };

  // 开启格式刷
  const toggleFormatBrush = (graph: Graph) => {
    graphRef.current = graph;
    const cell = graph?.getSelectedCells()?.find((item) => item.isNode());
    setEnableFormatBrush((state) => {
      if (!state) {
        const data = cell?.getData();
        formatBrushStyle.current = data;
        message.info("格式刷已开启");
        graph.on("cell:click", handleClick);
        graph.on("blank:click", handleClick);
      } else {
        formatBrushStyle.current = undefined;
        graph.off("cell:click", handleClick);
        graph.off("blank:click", handleClick);
      }
      return !state;
    });
  };

  // 撤销
  const onUndo = () => {
    graphRef.current?.undo();
  }

  // 重做
  const onRedo = () => {
    graphRef.current?.redo();
  }

  return {
    initMindMap,
    rightToobarActive,
    setRgithtToolbarActive,
    mindProjectInfo,
    setMindProjectInfo,
    canUndo,
    canRedo,
    onUndo,
    onRedo,
    enableFormatBrush,
    toggleFormatBrush,
  };
}
